###############################################################################
# Internet Mod Support
# by Josh Dahlby
###############################################################################
import sys
# Add the path for the socket.py file
sys.path.insert(0,"../../lib/pythonlib/plat-win")
import urllib
import string
import Reference

###############################################################################
# ProgressURL
#   Adds a progress hook to a download.
###############################################################################

class ProgressURL(urllib.FancyURLopener):

	def __init__(self, *args):
		apply(urllib.FancyURLopener.__init__, (self,) + args)

	def retrieve(self, url, filename, hook=None):
		url = urllib.unwrap(url)
		fp = self.open(url)
		try:
			headers = fp.info()
			tfp = open(filename, 'wb')
			try:
				bs = 1024*8
				clen = int(headers.getheader('content-length'))
				bc = 0
				if hook:
					hook(bc, bs, clen)
				block = fp.read(bs)
				while block:
					bc = bc + 1
					tfp.write(block)
					if hook:
						hook(bc, bs, clen)
					block = fp.read(bs)
			finally:
				tfp.close()
				del tfp
		finally:
			fp.close()
			del fp

###############################################################################
# WebMods
#   Manages download of mods on the web
###############################################################################

class WebMods:
	urlroot = "http://www.dahlby.net/games/blade/";
	mods = [];
	fetched = 0;
	
	def __init__(self, urlroot=None):
		if urlroot:
			self.urlroot = urlroot;

	###############################################################################
	# Public Methods
	###############################################################################

	# Get the mod info for a particular mod
	def GetModInfo(self, modname):
		self.checkFetch();
		for info in self.mods:
			if info['modname'] == modname:
				return info;
		return None;

	# Get the entire mod list
	def GetModList(self):
		self.checkFetch();
		return self.mods;

	# Get only the maps
	def GetMaps(self):
		self.checkFetch();
		return self.GetType('map');

	# Get only the mods
	def GetMods(self):
		self.checkFetch();
		return self.GetType('mod');

	# Get only 'type' mods
	def GetType(self, type):
		self.checkFetch();
		ret = []
		for info in self.mods:
			if info['modtype'] == type:
				ret.append(info)
		return ret

	# Get number of maps
	def GetMapCount(self):
		self.checkFetch();
		return self.GetTypeCount('map');

	# Get number of mods
	def GetModCount(self):
		self.checkFetch();
		return self.GetTypeCount('mod');

	# Get number of 'type'
	def GetTypeCount(self, type):
		self.checkFetch();
		count = 0
		for info in self.mods:
			if info['modtype'] == type:
				count = count + 1
		return count

	# Fetch the mod list from the server.  Returns 0 on failure and 1 on success
	def FetchMods(self):
		cmd = self.urlroot + "getmods.mv"
		try:
			url = urllib.urlopen(cmd)
			self.mods = []
			line = url.readline()
			while line != "":
				try:
					if len(line) >= 10:
						data = string.split(string.strip(line), ']][[')
						if len(data) >= 9:
							modinfo = {
								'modname': data[0],
								'modversion': data[1],
								'modauthor': data[2],
								'modwebsite': data[3],
								'modurl': data[4],
								'modsize': data[5],
								'modtype': data[6],
								'moddirname': data[7],
								'moddesc': ''
							}
							desc = string.split(data[8], '[[][]]')
							langdesc = {}
							for d in desc:
								idx = string.find(d, ':')
								if idx != -1:
									# Split it into lines 70 characters in length
									text = d[idx+1:]
									maxlen = 70;
									textlen = len(text)
									splittext = ""
									while textlen > 0:
										if len(splittext) > 0:
											splittext = splittext + "\n"
										temp = text[:maxlen];
										if textlen > maxlen:
											spaceidx = len(temp) - 1;
											while (spaceidx > 0) and (temp[spaceidx] not in string.whitespace):
												spaceidx = spaceidx - 1;
											if spaceidx == 0:
												spaceidx = maxlen;
											splittext = splittext + temp[:spaceidx]
											text = text[spaceidx+1:]
											textlen = len(text)
										else:
											splittext = splittext + temp
											textlen = 0
									langdesc[d[:idx]] = splittext
							modinfo['moddesc'] = langdesc
							self.mods.append(modinfo)
				except:
					Reference.debugprint("ERROR: Problem parsing mod info")
					Reference.debugprint(str(line))
					if Reference.DEBUG_INFO > 0:
						import traceback
						traceback.print_exc()
				line = url.readline()
		except:
			if Reference.DEBUG_INFO > 0:
				import traceback
				traceback.print_exc()
			self.mods = []
			self.fetched = 0
			return 0
		self.fetched = 1
		return 1

	# Get a mod from the web and unzips it.  Returns 0 on failure and 1 on success
	# Parameters:
	#   url = The url for the download
	#   dest = The destination for the file
	#   progressFunc = The progress function.  The progress methods receives one parameter which
	#              represents the percentage complete.
	def DownloadMod(self, url, dest, progressFunc=None):
		try:
			grabber = ProgressURL();
			try:
				grabber.retrieve(url, dest, progressFunc);
			finally:
				del grabber;
		except:
			if Reference.DEBUG_INFO > 0:
				import traceback
				traceback.print_exc()
			return 0
		return 1;

	###############################################################################
	# Private Methods
	###############################################################################

	# Make sure it is downloaded
	def checkFetch(self):
		if not self.fetched:
			self.FetchMods();

